use std::collections::HashMap;

use macroquad::prelude::*;
use serde::{Deserialize, Serialize};

use crate::{model::player::Player, screen::Screen};

#[allow(dead_code)]
#[derive(Serialize)]
#[serde(tag = "type")]
enum HostClientMessages {
    ResetAllBuzers,
    ResetPlayerBuzzer { user_id: u64 },
    UpdateScore { user_id: u64, diff: u32 },
}

#[derive(Deserialize)]
#[serde(tag = "type")]
enum HostServerMessages {
    Ack,
    UpdatePlayerStates { players: HashMap<u64, Player> },
}

#[derive(Default)]
pub struct HostScreen {
    players: HashMap<u64, Player>,
}

impl Screen for HostScreen {
    fn handle_frame(&mut self, ctx: &mut crate::context::Context) {
        clear_background(WHITE);

        draw_text(
            "I'm the host!",
            screen_width() / 2.0,
            screen_height() / 2.0,
            100.0,
            BLACK,
        );

        let button_size = screen_width() * 0.01;
        let button_location = Vec2::new(screen_width() * 0.50, screen_height() * 0.70);

        draw_circle(
            button_location.x,
            button_location.y,
            button_size * 1.1,
            DARKBROWN,
        );
        draw_circle(button_location.x, button_location.y, button_size, RED);
        if is_mouse_button_pressed(MouseButton::Left) {
            let loc = Vec2::from(mouse_position());
            let normalize = loc - button_location;

            if normalize.length() < button_size {
                ctx.send_msg(&HostClientMessages::ResetAllBuzers);
            }
        }
    }

    fn handle_messages(
        &mut self,
        ctx: &mut crate::context::Context,
    ) -> Option<crate::StateTransition> {
        if let Some(msg) = ctx.recv_msg() {
            match msg {
                HostServerMessages::Ack => {
                    // pass
                }
                HostServerMessages::UpdatePlayerStates { players } => {
                    self.players = players;
                }
            };
        }

        None
    }
}
